# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.



"""
The NetProfiler package offers a set of interfaces to control and work with
a SteelCentral NetProfiler appliance.
"""
from steelscript.netprofiler.core.netprofiler import *
from steelscript.netprofiler.core.report import *
